{**************************************************
RPC Broker Security Enhancement Example form      ver. 1.1
   Broker Development Team
   San Francisco IRM Field Office, Dept. of Veterans Affairs

Disclaimer:
   This example does not attempt to teach general Delphi and M programming.
   We intentionally removed any safeguards from the code that prevents
   passing values that are too small or too large.  Therefore, the important
   code remains uncluttered and the programmer is free to experiment and
   push the program beyond its limits.

Purpose:
   This sample application is an example of how to program Broker Security
   Enhancement (BSE) applications in Delphi and M using the RPC Broker.
   The demonstrated features include:
     - Connecting to an authenticating M server
     - Obtaining a user token with the 'XUS SET VISITOR' remote procedure
     - Connecting to a visited M server
     - Authenticating using application security phrase (in BseSample1.inc)
       plus token
     - Establishing a user context option
     - Using a remote procedure within the context option

   Original documentation for BSE was included in patch XWB*1.1*45. RPC Broker
   documentation is being updated to include BSE documentation.

Context option for this application:
   XWB BROKER EXAMPLE

Remote procedures used:
   XWB EXAMPLE ECHO STRING

Server M routine:
   XWBEXMPL

Current Release: Version 1.1 Patch 60
*************************************************************** }

{ **************************************************
  Changes in v1.1.60 (HGW 05/08/2014) XWB*1.1*60
  1. Recompiled to use Windows IPv4/IPv6 dual-mode APIs in
     patch XWB*1.1*60 RPC Broker components.

*************************************************}
unit fBseSample1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Trpcb;

type
  TForm1 = class(TForm)
    RPCB: TRPCBroker;
    AuthServer: TEdit;
    AuthPort: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    RemoteServer: TEdit;
    RemotePort: TEdit;
    edtPhrase: TEdit;
    edtReturned: TEdit;
    Label5: TLabel;
    btnTCPIP: TButton;
    btnEcho: TButton;
    btnM2M: TButton;
    btnDisconnect: TButton;
    lblEchoed: TLabel;
    btnHTTP: TButton;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    procedure btnTCPIPClick(Sender: TObject);
    procedure btnEchoClick(Sender: TObject);
    procedure btnM2MClick(Sender: TObject);
    procedure btnDisconnectClick(Sender: TObject);
    procedure btnHTTPClick(Sender: TObject);
  private
    { Private declarations }
  public
    procedure DoConnection(Key: String);
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

{$I BseSample1.inc}

procedure TForm1.btnTCPIPClick(Sender: TObject);
begin
  DoConnection(XwbKey1);
end;

procedure TForm1.btnEchoClick(Sender: TObject);
begin
  if not (edtPhrase.Text = '') then
  begin
    if not RPCB.Connected then
      ShowMessage('First connect by clicking on one of the Connect buttons')
    else
    begin
      RPCB.RemoteProcedure := 'XWB EXAMPLE ECHO STRING';
      RPCB.Param[0].Value := edtPhrase.Text;
      RPCB.Param[0].PType := literal;
      RPCB.Call;                           //execute RPC
      edtReturned.Text := RPCB.Results[0];  //for single value use Results[0]
    end;
  end
  else
    ShowMessage('No text in Phrase');
end;

procedure TForm1.DoConnection(Key: String);
var
  TokenValue: String;
begin
  RPCB.Server := AuthServer.Text;
  RPCB.ListenerPort := StrToInt(AuthPort.Text);
  RPCB.Connected := True;
  if RPCB.Connected then
  begin
    RPCB.RemoteProcedure := 'XUS SET VISITOR';
    RPCB.Call;
    TokenValue := RPCB.Results[0];
    RPCB.Connected := False;
    ShowMessage('Token: '+TokenValue);
    if not (TokenValue = '') then
    begin
      RPCB.Server := RemoteServer.Text;
      RPCB.ListenerPort := StrToInt(RemotePort.Text);
      RPCB.SecurityPhrase := Key + '^' + TokenValue;
      RPCB.Connected := True;
      if RPCB.Connected then
      begin
        ShowMessage('Signed  on to Remote Server');
        RPCB.CreateContext('XWB BROKER EXAMPLE');
        btnDisconnect.Enabled := True;
        btnEcho.Enabled := True;
        btnM2M.Enabled := False;
        btnTCPIP.Enabled := False;
      end
      else
        ShowMessage('Connection to Remote Server failed!');
    end;
  end
  else
    ShowMessage('Initial Sign-on Failed');
end;

procedure TForm1.btnM2MClick(Sender: TObject);
begin
  DoConnection(XwbKey2);
end;

procedure TForm1.btnDisconnectClick(Sender: TObject);
begin
  RPCB.Connected := False;
  btnDisconnect.Enabled := False;
  btnEcho.Enabled := False;
  btnM2M.Enabled := True;
  btnTCPIP.Enabled := True;
  btnHTTP.Enabled := True;
end;

procedure TForm1.btnHTTPClick(Sender: TObject);
begin
  RPCB.Server := RemoteServer.Text;
  RPCB.ListenerPort := StrToInt(RemotePort.Text);
  RPCB.SecurityPhrase := XwbKey3 + '^XWBHDL851-487411_0';
  RPCB.Connected := True;
  if RPCB.Connected then
  begin
    ShowMessage('Signed  on to Remote Server');
    RPCB.CreateContext('XWB BROKER EXAMPLE');
    btnDisconnect.Enabled := True;
    btnEcho.Enabled := True;
    btnM2M.Enabled := False;
    btnTCPIP.Enabled := False;
    btnHTTP.Enabled := False;
  end
  else
    ShowMessage('Connection to Remote Server failed!');
end;

end.
